/*


  topfcomm

  send and receive encoder/decoder for the function via seriallink communication
  every TAP function should have a corresponding communication function here

  every TAP function consists of following communication functions:

  CTAP_[functionname]						- command word for tap
  STAP_[functionname](...parameters...)		- send tap command
  ETAP_[functionname](...parameters...)		- execute tap command (topfield only, execution of the command and answering)
  ATAP_[functionname](...parameters...)		- decode answer of tap command


  a single function with return parameters generates following sequence:

  PC: allocating buffer (or use preallocated maxbuffer)
  PC: generate tapcommand (commandword CTAP_) buffer (via STAP_) 
  PC: send buffer over serial line
  TP: receives command, allocates memory according to length, or use preallocated buffer
  TP: decodes command word (CTAP_) and call ETAP_. ETAP_ generates the answerbuffer
  TP: send the answerbuffer to the PC
  PC: receives the answercommand (with CTAP_ as commandword) and decodes the parameters of the answer (or no parameters if setonly)


*/

#include "common_definitions.h"

#define RUN_TOPF
#ifdef _WIN32
	#undef		RUN_TOPF
#endif

// static buffer definition for input and output buffers
CHAR ib[0x10000];
CHAR ob[0x10000];



#ifdef RUN_TOPF

void SendBuffer()
{
}

void ReceiveBuffer()
{
}

void DecodePackage()
{
	WORD cmd;
	ReceiveBuffer();
	GetWord(ib, 1, &cmd);

	switch (cmd) {
		case CTAP_GetState	: ETAP_GetState(); break;
		default:
			// here send error code
			break;
	}
}

#else // RUN ON WINDOWS

void SendBuffer()
{
	WORD len;
	WORD pos = GetWord(ob, 1, &len);
	glob_ser.Write(ob, len + 1 /*stx*/+ 2 /*commandcode*/);
}

void ReceiveBuffer()
{
	CHAR stx = 0;
	CHAR hi, lo;
	WORD len = 0;
	do {
		glob_ser.Read(&stx, 1); 
	} while (stx == 0x02);

	ib[0] = stx;

	glob_ser.Read(&hi, 1); // length of telegram
	glob_ser.Read(&lo, 1); 
	len = (hi << 8) + lo;

	ib[1] = hi;
	ib[2] = lo;

	for (WORD loop = 0; loop < len; loop++) {
		glob_ser.Read(&ib[loop], 1);
	}

}

#endif



/*
#define CTAP_						
void STAP_()
{
	CreateMessage(&ob, CTAP_);
}

void ATAP_()
{
	int pos = 1; // position after the start of text code
	WORD len;
	// get length of telegram
	pos = GetWord(&ib, pos, &len);
	//get return parameter
	//get variable parameters
}
*/




void STAP_GetState()
{
	CreateMessage(ob, CTAP_GetState);
}

void ATAP_GetState(LONG *state, LONG *subState)
{
	int pos = 1; // position after the start of text code
	WORD len;
	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
	pos = GetLong(ib, pos, state);
	pos = GetLong(ib, pos, subState);
}

#ifdef RUN_TOPF
void ETAP_GetState()
{
	int pos = 1;
	WORD len;
	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
	//execute the topf command
	LONG p1;
	LONG p2;
	TAP_GetState(&p1, &p2);
	CreateMessage(ob, CTAP_GetState);
	AddLongParm(ob, p1);
	AddLongParm(ob, p2);
	SendBuffer();
}
#else // RUN ON WINDOWS
void TAP_GetState(LONG *state, LONG *subState)
{
	STAP_GetState();
	SendBuffer();
	ReceiveBuffer();
	ATAP_GetState(state, subState);
}
#endif


void STAP_ExitNormal()
{
	CreateMessage(ob, CTAP_ExitNormal);
}

void ATAP_ExitNormal()
{
	int pos = 1; // position after the start of text code
	WORD len;
	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
}

void STAP_EnterNormal()
{
	CreateMessage(ob, CTAP_EnterNormal);
}

void ATAP_EnterNormal()
{
	int pos = 1; // position after the start of text code
	WORD len;
	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
}

void STAP_Exit()
{
	CreateMessage(ob, CTAP_Exit);
}

void ATAP_Exit()
{
	int pos = 1; // position after the start of text code
	WORD len;
	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
}


void STAP_GetSystemVar(int varid)
{
	CreateMessage(ob, CTAP_GetSystemVar);
	AddWordParm(ob, varid);
}

int ATAP_GetSystemVar()
{
	int pos = 1; // position after the start of text code
	WORD len;
	WORD back;

	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	pos = GetWord(ib, pos, &back);
	//get variable parameters

	return back;
}


void STAP_SetSystemVar(int varid, int value)
{
	CreateMessage(ob, CTAP_SetSystemVar);
	AddWordParm(ob, varid);
	AddWordParm(ob, value);
}

int ATAP_SetSystemVar()
{
	int pos = 1; // position after the start of text code
	WORD len;
	WORD back;

	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	pos = GetWord(ib, pos, &back);
	//get variable parameters

	return back;
}

void STAP_WriteSystemVar()
{
	CreateMessage(ob, CTAP_WriteSystemVar);
}

int ATAP_WriteSystemVar()
{
	int pos = 1; // position after the start of text code
	WORD len;
	WORD back;

	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	pos = GetWord(ib, pos, &back);
	//get variable parameters

	return back;
}

void STAP_GenerateEvent(WORD eventcode, LONG param1, LONG param2)
{
	CreateMessage(ob, CTAP_GenerateEvent);
	AddWordParm(ob, eventcode);
	AddLongParm(ob, param1);
	AddLongParm(ob, param2);
}

void ATAP_GenerateEvent()
{
	int pos = 1; // position after the start of text code
	WORD len;

	// get length of telegram
	pos = GetWord(ib, pos, &len);
	//get return parameter
	//get variable parameters
}
