/*


  topfcomm

  send and receive encoder/decoder for the function via seriallink communication
  every TAP function should have a corresponding communication function here

  every TAP function consists of following communication functions:

  CTAP_[functionname]						- command word for tap
  STAP_[functionname](...parameters...)		- send tap command
  ETAP_[functionname](...parameters...)		- execute tap command (topfield only, execution of the command and answering)
  ATAP_[functionname](...parameters...)		- decode answer of tap command


  a single function with return parameters generates following sequence:

  PC: allocating buffer (or use preallocated maxbuffer)
  PC: generate tapcommand (commandword CTAP_) buffer (via STAP_)
  PC: send buffer over serial line
  TP: receives command, allocates memory according to length, or use preallocated buffer
  TP: decodes command word (CTAP_) and call ETAP_. ETAP_ generates the answerbuffer
  TP: send the answerbuffer to the PC
  PC: receives the answercommand (with CTAP_ as commandword) and decodes the parameters of the answer (or no parameters if setonly)


*/

#include "common_definitions.h"
#include "tap_definitions.h"

#define RUN_TOPF
#ifdef _WIN32
	#undef		RUN_TOPF
#else
#include "tap.h"
#include "tap_topfield.h"
#endif


#define STX_TIMEOUT					1000
#define CHAR_TIMEOUT				500
#define OVERALL_TIMEOUT				3000


// static buffer definition for input and output buffers
CHAR ib[0x10000];
CHAR ob[0x10000];
CHAR ob2[0x20002];


//void ETAP_GetState();
//void STAP_GetState();
//void ATAP_GetState(LONG *state, LONG *subState);


WORD ___asctohex[] =
{
  0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,
  0xAA,0xBB,0xCC,0xDD,0xEE,0xFF,
  0x00,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF
};

/****************************************************************************
 * Converts first to string elements to a unsigned char
 ****************************************************************************/

CHAR ConvAscHex(CHAR c1, CHAR c2)
{
	return (___asctohex[c1-0x30] & 0xf0) +	(___asctohex[c2-0x30] & 0x0f);
}


#define	TRACECHAR(x)	{if (((x) >= ' ') && ((x) < 0xA5)) TRACE("%c", x); else TRACE(" 0x%02X ", (int)(x)&0xFF);}
#define	TRACECHARSEND(x)	{TRACE("%02X", (int)(x)&0xFF);}

#define CHECKCHAR(x) glob_ser.Read(&i, 1, 0, 0, CHAR_TIMEOUT); if (i != x) return -1;

int SendBuffer()
{
	WORD len;
	WORD pos = GetWord(ob, 1, &len);
	CHAR bb[2];
	CHAR i;
	LONG l = 0;

	TRACE("SEND:");
	len += 3;

	//ob2[l++] = STX_CHAR;
	glob_ser.Write(&(ob[0]), 1); CHECKCHAR(ob[0]);
	
	TRACE("STX_CHAR| ");
	for (int loop = 1; loop < len; loop++) {
		CString s; s.Format("%02X", ob[loop]);
		bb[0] = s[0]; bb[1] = s[1];
		TRACECHAR(bb[0]);
		TRACECHAR(bb[1]);

		glob_ser.Write(&(bb[0]), 1); CHECKCHAR(bb[0]);
		glob_ser.Write(&(bb[1]), 1); CHECKCHAR(bb[1]);
		
		ob2[l++] = bb[0]; 
		ob2[l++] = bb[1]; 
	}
	//ob2[l++] = 0x03;
	
	//glob_ser.Write(ob2, l);
	TRACE("\n");
	return 0;
}

int SendBreak() 
{
	CHAR br[2];
	CHAR i;
	br[0] = BREAK_CHAR;
	br[1] = '\0';

	glob_ser.Write(br, 1); TRACECHAR(br[0]);
	CHECKCHAR(br[0]);
	//glob_ser.Write(br, 1); TRACECHAR(br[0]);
	//glob_ser.Write(br, 1); TRACECHAR(br[0]);
	//SleepEx(100L, FALSE);
	return 0;
}


#define CHECK_TIMEOUT() if ((GetCurrentTimeMilliSeconds() - timer) > OVERALL_TIMEOUT) { \
							TRACE("TIMEOUT\n"); \
							return -1; \
						}


// returns -1 if receive timeout
int ReceiveBuffer()
{
	CHAR stx = 0;
	CHAR hi, lo;
	CHAR bb[2];
	WORD len = 0;

	UINT64 timer = GetCurrentTimeMilliSeconds();

	TRACE("DIRECT:");
	do {
		glob_ser.Read(&stx, 1, 0, 0, STX_TIMEOUT);
		TRACECHAR(stx);

		CHECK_TIMEOUT();

	} while (stx != STX_CHAR);

	TRACE("* ");

	ib[0] = stx;

	glob_ser.Read(bb, 2, 0, 0, CHAR_TIMEOUT);  hi = ConvAscHex(bb[0], bb[1]);
	TRACE("%c%c", bb[0], bb[1]);

	CHECK_TIMEOUT();

	glob_ser.Read(bb, 2, 0, 0, CHAR_TIMEOUT);  lo = ConvAscHex(bb[0], bb[1]);
	TRACE("%c%c", bb[0], bb[1]);

	CHECK_TIMEOUT();

	len = hi;
	len = (len << 8) + lo;

	ib[1] = hi;
	ib[2] = lo;

	TRACE(" (len:%d) ", len);

	for (WORD loop = 0; loop < len; loop++) {
		glob_ser.Read(bb, 2, 0, 0, CHAR_TIMEOUT);  
		ib[loop + 3] = ConvAscHex(bb[0], bb[1]);
		
		TRACE("%c%c", bb[0], bb[1]);
		
		CHECK_TIMEOUT();
	}
	TRACE(" >> OK\n");

	return 0;
}

